require 'test_helper'

feature "Item Proofreading" do

  before do

    @post = create(:not_translated_post)
    @german_title = "Mein erster Blogpost"
    @german_body = "Ich liebe Hummus"
    translation_params = { title: @german_title, body: @german_body }
    user = create(:bob_the_translator)
    translation_state_by_locale(@post, :de).save_translation_version translation_params, user

  end

  scenario "Proofreader approves translation" do

    login_as create(:michael_the_proofreader)

    visit root_path
    click_link "Proofread"

    assert page.has_content? @german_title
    assert page.has_content? @german_body

    click_button "Approve"

    assert page.has_content? "Nothing left to proofread"
    assert_translation_approved

  end

  scenario "Proofreader rejects translation" do

    login_as create(:michael_the_proofreader)

    visit root_path
    click_link "Proofread"

    assert page.has_content? @german_title
    assert page.has_content? @german_body

    rejection_comment = "Not good enough"
    fill_in "comment-area", with: rejection_comment
    # byebug
    click_button "Reject"

    assert page.has_content? "Nothing left to proofread"
    assert_translation_rejected rejection_comment

    # # make sure translator sees the comment
    # bob = translation_state_by_locale(@post, :de).user
    # login_as bob
    # visit root_path

    # click_link "Translate"

    # assert page.has_content? rejection_comment

  end

  scenario "Proofreader stops proofreading" do

    login_as create(:michael_the_proofreader)
    translator = translation_state_by_locale(@post, :de).user

    visit root_path
    click_link "Proofread"

    click_button "Stop Proofreading"

    @post.reload
    saved_translation_state = translation_state_by_locale @post, :de

    # make sure translation state is saved correctly
    assert_equal :done, saved_translation_state.status
    assert_equal translator, saved_translation_state.user

  end

  scenario "Admin approves a translation through a translation state page" do

    find_and_open_translation_state_as_admin

    click_button "Approve"

    assert_translation_approved

  end

  scenario "Admin rejects a translation through a translation state page" do

    find_and_open_translation_state_as_admin

    rejection_comment = "Not good enough"
    fill_in "comment-area", with: rejection_comment
    click_button "Reject"

    assert_translation_rejected rejection_comment

  end

  private

  def find_and_open_translation_state_as_admin
    login_as create(:johnny_the_admin)

    visit root_path
    click_link "Translations Queue"

    # should have 1 done translation
    click_link "Done (1)"

    german_translation_state = translation_state_by_locale @post, :de
    find(:css, "a[href='#{translation_state_path(german_translation_state.id)}']").click

    assert page.has_content? @german_title
    assert page.has_content? @german_body
  end

  def assert_translation_approved
    @post.reload
    saved_translation_state = translation_state_by_locale @post, :de

    # make sure translation state is saved correctly
    assert_equal :approved, saved_translation_state.status
    assert_equal User.current_user, saved_translation_state.user
    
    # translation should be still saved locally
    saved_translation_data = saved_translation_state.translation_data
    assert_not_nil saved_translation_data
    assert_not_empty saved_translation_data

    assert_equal @german_title, saved_translation_data[:title]
    assert_equal @german_body, saved_translation_data[:body]

    # actual post should be updated
    assert_equal @german_title, @post.title_de
    assert_equal @german_body, @post.body_de
  end

  def assert_translation_rejected rejection_comment
    translation_state_before_change = translation_state_by_locale @post, :de
    @post.reload
    saved_translation_state = translation_state_by_locale @post, :de

    # making sure the translation state and its versions were properly updated
    assert_equal :pending, saved_translation_state.status
    assert_equal translation_state_before_change.user, saved_translation_state.user

    rejection_version = saved_translation_state.versions.order(id: :asc)[-2]
    assert_equal :rejected, rejection_version.status
    assert_equal User.current_user, rejection_version.user
    # assert_equal rejection_comment, rejection_version.comment
    
    # translation should be still saved locally
    saved_translation_data = saved_translation_state.translation_data
    assert_not_nil saved_translation_data
    assert_not_empty saved_translation_data

    assert_equal @german_title, saved_translation_data[:title]
    assert_equal @german_body, saved_translation_data[:body]

    # actual post shouldn't be updated
    assert_not_equal @german_title, @post.title_de
    assert_not_equal @german_body, @post.body_de
  end

end